function ApuracaoEfetivacaoController(General, ScreenService, MessageService, templateManager, maskEngine, ZhUtil, ZHPromise, Utilities, Cmp03900_LancamentoAutorizacaoDeFornecimento_Service, metaDataFactory, restEngine, ZhFilterPreferences) {
     /*global Constants*/
    var ServiceLancamentoAF = Cmp03900_LancamentoAutorizacaoDeFornecimento_Service;
    let param;
    let index;
    let produtos = [];
    let idjustificativa;
    let podeAbrirInformacao;
    let messages = {
        errosApuracao: "Erros na Apuração",
        timeout: "Tempo limite de conexão excedido."
    };
    let allEntregas;

    /*************************************************************************************/
    /*                           FUNÇÕES GERAIS E DE VALIDAÇÃO                           */
    /*************************************************************************************/

    /*
     * Define a "Central de Filial" e os parâmetros para visualizção de componentes da tela.
     * A visualização campo 'SCCOMPRAS' do filtro e as informações de 'Produtos' 
     * no Stripe.
     */
    this.getParams = function() {
        General.callRestEngine("cmp_fornecimento_autorizacaoFornecimento#/getParams", "Row", {
            "CDFILIAL": getLocalVar("CDFILIAL")
        }).then(function(response) {

            this.idjustificativa = response.dataset.getParams[1][0].IDEXJUSTESCM;
            var idsisdos = response.dataset.getParams[1][0].IDSISDOS;
            var paramApurExtra = response.dataset.getParams[1][0].IDAPUREFESOLEXT;
            var paramProd = response.dataset.getParams[2][0].IDCONSPRSOL;

            var widgetFilter = templateManager.container.getWidget('FILTER');
            var vrunitario = templateManager.container.getWidget('widgetOldCot').getField('VRPRECUNIAP');
            var widgetStripe = templateManager.container.getWidget('STRIPE');

            setLocalVar('NMFILCENTCMP', response.dataset.getParams[0][0].NMFILCENTCMP);
            setLocalVar('CDFILCENTCMP', response.dataset.getParams[0][0].CDFILCENTCMP);

            if (paramApurExtra == 'N') {
                widgetFilter.getField('EXTRA').isVisible = false;
                widgetFilter.getField('EXTRA').validations = {
                    'required': false
                };
            }

            if (idsisdos == 3) {
                vrunitario.inlineEditable = false;
            }

            this.openFilter();
        }.bind(this));
    };

    this.getRow = function(widget) {
        return angular.copy(widget.currentRow);
    };

    this.removeClass = function(element, className) {
        $(element).removeClass(className);
    };

    this.removeColor = function(args) {
        var row = this.getRow(args);
        args.owner.widget.setCurrentRow(row);
        this.removeClass('.lineImportant', 'currentRow');
    };
    
    this.backFromSwipe = function() {
        ScreenService.closeSwipe();
    };
    
    this.backFromPopup = function() {
        ScreenService.closePopup();
    };
    
    /**
     * Retorna a concatenação do array passado pro parâmetro. 
     * @param {Array} arr - array a ser concanetado.
     * @param {string} defaultValue - Valor a ser retornado caso o arr esteja
     * vazio ou em branco.
     * @returns {string} Array concatenado ou o defaultValue passado por
     * parâmetro. Se não houver defaultValue, retorna 'T'.
     */
    function concatArrayForParams(arr, defaultValue = 'T') {
       return !Util.isEmptyOrBlank(arr) ? arr.join('_') : defaultValue;
    }

    function getEventFromWidget(widget, eventName) {
        return widget.events.filter(event => { return event.name === eventName; })[0];
    }
    
    function setCodeOnEventWidgetAfterMoveRow(eventCode) {
        const event = getEventFromWidget(templateManager.container.getWidget('widgetOldCot'), 'WidgetAfterMoveRow');
        
        event.code = eventCode;
    }
    
    this.removeOpenPorEntregaFromWidgetAfterMoveRow = function() {
        const eventCode = "ApuracaoEfetivacaoController.removeClass('.lineImportant', 'currentRow');";
        
        setCodeOnEventWidgetAfterMoveRow(eventCode);
    };
    
    this.addOpenPorEntregaOnWidgetAfterMoveRow = function() {
        const eventCode = "ApuracaoEfetivacaoController.openPorEntrega();ApuracaoEfetivacaoController.removeClass('.lineImportant', 'currentRow');";
        
        setCodeOnEventWidgetAfterMoveRow(eventCode);
    };
    
    function isActionOfInlineEdit(action) {
        return action.showAsAction === 'back_inline_edit' || action.showAsAction === 'never_inline_edit';
    }
    
    function turnActionVisible(action) {
        action.isVisible = true;
    }
    
    function getEditableFields(widget) {
		return widget.fields.filter(function(field) {
			return field.inlineEditable;
		});
	}
    
    function actionsAfterInlineEditOperation(widgetCotacao) {
        widgetCotacao.inlineEditing = false;
		widgetCotacao.list();
		widgetCotacao.actions.filter(isActionOfInlineEdit).map(turnActionVisible);
    }
    
    function removeChangedPropertiesFromRow(widgetCotacao) {
        let editableFields = getEditableFields(widgetCotacao);
        
		widgetCotacao.dataSource.data.forEach(function(row) {
			editableFields.forEach(function(field) {
				if (row.hasOwnProperty('changed_' + field.name)) {
					delete row['changed_' + field.name];
				}
			});
			delete row.changed;
		});
    }
    
    function transferRowData(originRow, destinyRow) {
        for (let property in originRow) {
			destinyRow[property] = originRow[property];
		}
    }
    
    this.permiteExecutarOpenInformacao = function() {
        podeAbrirInformacao = true;
    };
    
    /**
     * Função responsável por simular o funcionamento que evita que um evento
     * específico do WidgetAfterMoveRow fosse acionado ao clicar no field.
     * 
     * Originalmente, este recurso foi implementado utilizando a seguinte composição de eventos:
     * - FieldOnClick do field VRPRECUNIAP, acionando a função removeEventFromWidgetAfterMoveRow;
     * - WidgetBeforeInlineEditMoveRow da widget widgetOldCot, acionando a função addEventInWidgetAfterMoveRow.
     * 
     * Para simular, foi utilizada uma promise.
     */
    this.changeEventCodeWidgetAfterMoveRow = function() {
        let deferred = ZHPromise.defer();
        
        setCodeOnEventWidgetAfterMoveRow("ApuracaoEfetivacaoController.updateApuracao()");
        deferred.resolve();
        
        return deferred.promise.then(() => { this.addOpenPorEntregaOnWidgetAfterMoveRow(); });
    };
    
    /*************************************************************************************/
    /*                                 FUNÇÕES DO FILTRO                                 */
    /*************************************************************************************/

    this.openLoginValidation = function(widget) {
        if (widget.currentRow.CDCOMPRADOR != "") {

            General.callRestEngine("cmp_fornecimento_apuracaoEfetivacao#/compValidation", "Row", {
                "CDCOMPRADOR": widget.currentRow.CDOPERADOR,
                "CDOPERADOR": getLocalVar('CDOPERADOR'),
                "CDCENTCMP": getLocalVar('CDFILCENTCMP')
            }).then(function(response) {
                if (response.dataset.passwordAut.PARAM === 'S') {
                    widget.container.getWidget('LOGIN').currentRow.CDCOMPRADOR = widget.currentRow.CDCOMPRADOR;
                    widget.container.getWidget('LOGIN').currentRow.CDOPERADOR = widget.currentRow.CDOPERADOR;
                    ScreenService.openPopup(widget.container.getWidget('LOGIN'));
                }
            }).catch(function(error) {
                ScreenService.showMessage(error.data ? error.data.error : messages.timeout, 'error', 'error');
                widget.currentRow.CDCOMPRADOR = "";
            });
        }
    };

    this.loginValidation = function(widget) {
        if(!Util.isEmptyOrBlank(widget.currentRow.PASSWORD)){
            General.callRestEngine("cmp_fornecimento_apuracaoEfetivacao#/passwordValidation", "Row", widget.currentRow).then(function(response) {

                widget.currentRow = {};
                ScreenService.closePopup();
    
            }).catch(function(error) {
    
                ScreenService.showMessage(error.data.error);
                widget.currentRow.PASSWORD = "";
            });    
        } else {
            MessageService.showMessageByCode("INFORME_SENHA");
        }
    };

    this.openFilter = function() {
        var popupWidget = templateManager.container.getWidget('FILTER');
        
        popupWidget.getField('DTENTREGA').defaultValue = {
            start: moment().format('DD/MM/YYYY'),
            end: moment().format('DD/MM/YYYY')
        };
        
        ServiceLancamentoAF.retornaComprador().then(function(callback) {
            const comprador = callback.dataset.comprador_operador;
            if (comprador.length) {
                const codigoComprador = comprador[0].CDOPERADOR;
                const nomeComprador = comprador[0].NMOPERADOR;
                const fieldComprador = popupWidget.getField('CDCOMPRADOR');
                fieldComprador.reload().then(() => {
                    ScreenService.openPopup(popupWidget);
                    popupWidget.newRow();
                    popupWidget.currentRow.CDOPERADOR = codigoComprador;
                    popupWidget.currentRow.CDCOMPRADOR = codigoComprador + ' | ' + nomeComprador;
                    if (popupWidget.inEditMode()) {
                        popupWidget.newRow = function() {}; //Faz o newRow ser executado apenas uma vez
                    }
                });
            } else {
                ScreenService.openPopup(popupWidget);
                popupWidget.newRow();
                if (popupWidget.inEditMode()) {
                    popupWidget.newRow = function() {};
                }
            }
        }, function(err) {
            ScreenService.showMessage(err.data.error);
            ScreenService.openPopup(popupWidget);
            popupWidget.newRow();
            if (popupWidget.inEditMode()) {
                popupWidget.newRow = function() {};
            }
        });
    };
    
    function isProdutoValido(currentRow) {
        return General.comparaPratoProduto(currentRow.CDPRODINI, currentRow.CDPRODFIN, getLocalVar('CDPICTPROD'));
    }

    /* Apura os produtos de acordo com os parâmetros passados no popup.
     * 
     * Seta os campos do stripe com os parâmetros do primeiro produto a ser exibido.
     * Defini os parâmetros locais a serem passados na requisição.
     */
    this.applyFilter = function(widget) {
        if (widget.isValid()) {
            if (isProdutoValido(widget.currentRow)) {
                widget.container.getWidget('STRIPE').currentRow = {
                    'CDPRODUTO': "",
                    'NMPRODUTO': "",
                    'NMOPERADOR': widget.currentRow.CDCOMPRADOR,
                    'DTINI': widget.currentRow.DTENTREGA.start,
                    'DTFIN': widget.currentRow.DTENTREGA.end
                };
                
                widget.currentRow.FILIPROC = getLocalVar('CDFILIAL'); //FILIAL LOGADA
                widget.currentRow.CDOPERADORLOG = getLocalVar('CDOPERADOR'); //OPERADOR LOGADO
                widget.currentRow.CDFILIAL = getLocalVar('CDFILIAL'); //FILIAL LOGADA
    
                this.loadProdutos(widget);
            } else {
                MessageService.showMessageByCode("PRODUTO_INICIAL_MENOR_IGUAL_FINAL");
            }
        }
    };
    
    this.limpaCampoUnidades = function() {
        templateManager.container.getWidget('FILTER').getField('LISTAFILIAL').clearValue();
    };

    function updateConditionOnDataSourceFilter(params) {
        const { field, widget, conditionName, operator, newValue, cleaningOperation } = params;

        if (!Util.isEmptyOrBlank(widget.currentRow[conditionName])) {
            const regionCondition = field.dataSourceFilter.filter(filtro => filtro.name == conditionName);

            if (regionCondition.length) {
                regionCondition[0].value = newValue;
            } else {
                field.dataSourceFilter.push({
                    name: conditionName,
                    operator,
                    value: newValue,
                });
            }
        } else {
            cleaningOperation(field, conditionName);
        }
    }

    /* Define as regras para o filtro do campo "Unidades" de acordo
     * com os campos "Regiao" e "Agrupamento de Unidades".
     */
    this.filterUnidades = function(widget) {
        const field = widget.getField('LISTAFILIAL');
        const params = {
            field,
            widget,
        };
        
        params.conditionName = 'CDREGIAO';
        params.operator = 'IN';
        params.newValue = widget.currentRow.CDREGIAO;
        params.cleaningOperation = (field, conditionName) => field.dataSourceFilter = field.dataSourceFilter.filter(filtro => filtro.name != conditionName);
        
        updateConditionOnDataSourceFilter(params);
        
        params.conditionName = 'CDGRUPFILI';
        params.operator = '=';
        params.newValue = concatArrayForParams(widget.currentRow.CDGRUPFILI);
        params.cleaningOperation = (field, conditionName) => {
            const condition = field.dataSourceFilter.filter(filtro => filtro.name == conditionName)[0];

            condition.value = 'T';
        };
        
        field.reload();
    };
    
    function resetFieldProduto(field) {
        field.readOnly = true;
		field.validations = {};
		deepClearField(field);
		if (field.widget.currentRow.$error) {
        	delete field.widget.currentRow.$error[field.name];
		}
    }
    
    function activateFieldProduto(field) {
        field.readOnly = false;
		field.validations = {
			required: true
		};
    }
    
    this.onChangeTodosProdutosAComprar = function() {
        const widgetFilter = templateManager.container.getWidget('FILTER');
        const fieldTodosProdutos = widgetFilter.getField('TODOS_PRODUTOS');
        const fieldProdutoInicial = widgetFilter.getField('PRODUTOINI');
        const fieldProdutoFinal = widgetFilter.getField('PRODUTOFIN');
        
        if (fieldTodosProdutos.checked) {
			resetFieldProduto(fieldProdutoInicial);
			resetFieldProduto(fieldProdutoFinal);
		} else {
			activateFieldProduto(fieldProdutoInicial);
			activateFieldProduto(fieldProdutoFinal);
		}
    };

    this.nextProduto = function() {
        if (index != produtos.length - 1) {
            index++;
            this.setCotacaoWithParams();
        }
    };

    this.previousProduto = function() {
        if (index != 0) {
            index--;
            this.setCotacaoWithParams();
        }
    };
    
    function setVisibleActions(widget, isVisible, exceptions = []) {
        widget.actions
            .filter(action => !exceptions.includes(action.name))
            .forEach(action => action.isVisible = isVisible);
    }

    /*************************************************************************************/
    /*                                 FUNÇÕES 'PRODUTOS'                                */
    /*************************************************************************************/

    /* Roda procedure que retorna dois valores para parametrização dos produtos
     * que serão apurados e em seguida faz a requisição dos produtos de acordo
     * com os parametros recebidos.
     */
    this.loadProdutos = function(widget) {
        ScreenService.confirmMessage("Confirma apuração de dados para efetivação de AF's?",  'question', function(success) {
            let row = angular.copy(widget.container.getWidget('FILTER').currentRow);
        
            row.CDPRODINI = row.CDPRODINI || '';
            row.CDPRODFIN = row.CDPRODFIN || '';
            
            General.callRestEngine("cmp_fornecimento_apuracaoEfetivacao#/apuracao", "Row", row).then( function(response) {
                let widgetCotacao = templateManager.container.getWidget('widgetOldCot');
                let widgetStripe  = templateManager.container.getWidget('STRIPE');
                
                param = response.dataset.param[0].P_NRSEQAPUR;
                
                General.callRestEngine("cmp_fornecimento_apuracaoEfetivacao#/qryAutGrupOp", "Row", { 'CDOPERADOR': getLocalVar("CDOPERADOR") }).then(function(response) {
                    if (response.dataset.qryAutGrupOp.PARAM != 'S') {
                        MessageService.showMessageByCode("OPR_SEM_PERMISSAO_ESC_PRECO");
                        widgetCotacao.getField('IDITMSELAPUR').events = [];
                    }
                }.bind(this));
    
                General.callRestEngine("cmp_fornecimento_apuracaoEfetivacao#/crud/adqProduto", "FilterData", [
                    {
                        'name': 'P_CDCENTCMP',
                        'operator': 'IN',
                        'value': getLocalVar('CDFILCENTCMP')
    				},
                    {
                        'name': 'P_NRSEQAPUR',
                        'operator': 'IN',
                        'value': param
    				}
    			]).then(function(response) {
                    produtos = response.dataset.adqProduto;
                    index = 0;
                    this.setCotacaoWithParams();
                }.bind(this));
    
                setVisibleActions(widgetCotacao, true);
                
                widgetStripe.actions.forEach(function(value, index) {
                    widgetStripe.actions[index].isVisible = true;
                }.bind(this));
                
                ScreenService.closePopup();
    
            }.bind(this)).catch(function(error) {
                let errosEncontrados = error.data;
                let mensagem = 'Não há dados a serem listados para essa apuração.';
                
                if (Array.isArray(errosEncontrados.error)) {
                    if (Util.isEmptyOrBlank(errosEncontrados.error)) {
                        mensagem = mensagem.concat(ScreenService.i18n("Favor verificar o relatório de Produtos não Comprados"));
                    }
                } else {
                    mensagem = errosEncontrados.error;
                }
                
                this.confirmMessageModified(mensagem, error).then(
                    function() {},
                    function(callback) {
                        this.handleInsertErrors(callback);
                    }.bind(this));
            }.bind(this));
        }.bind(this), function(error) {});
    };

    /* Seta no stripe, o produto selecionado.
     */
    this.setCotacaoWithProdutoOnGrid = function(widget) {
        var cdprod = widget.container.getWidget('PRODUTOS').currentRow.CDPRODUTO;

        produtos.forEach(function(value, i) {
            if (cdprod == produtos[i].CDPRODUTO) {
                index = i;
            }
        }.bind(this));
        this.setCotacaoWithParams();
        ScreenService.closePopup();
    };

    this.openPopupProdutos = function() {
        var widProd = templateManager.container.getWidget('PRODUTOS');

        widProd.dataSource.setData(produtos);
        ScreenService.openPopup(widProd);
    }; 

    /*************************************************************************************/
    /*                                 FUNÇÕES COTAÇÃO                                   */
    /*************************************************************************************/
    
    /* Faz uma requisição em 'Cotação' filtrando com o produto selecionado
     * no 'STRIPE'.
     */
    function getCotacoes(fillFilterFunction) {
        templateManager.container.getWidget('STRIPE').currentRow.CDPRODUTO = produtos[index].CDARVPROD;
        templateManager.container.getWidget('STRIPE').currentRow.NMPRODUTO = produtos[index].NMPRODUTO;
        
        return General.callRestEngine("cmp_fornecimento_apuracaoEfetivacao#/adqCotacao", "Row", {
            'P_CDCENTCMP': getLocalVar('CDFILCENTCMP'),
            'P_CDPRODESTO': produtos[index].CDPRODUTO,
            'P_NRSEQAPUR': param,
            'P_CDFILIOPELOG': getLocalVar('CDFILIAL')
        }).then(fillFilterFunction).catch(function(error) {
            ScreenService.showMessage(error.data ? error.data.error : messages.timeout, 'error', 'error');
        }); 
    }
    
    function setCotacao(response) {
        const widgetCotacao = templateManager.container.getWidget('widgetOldCot');
            
        widgetCotacao.dataSource.setData(response.dataset.cotacao[0]);
        widgetCotacao.reload();
    }
    
    this.setCotacaoWithParams = function() {
        return getCotacoes(function(response) {
            const widgetCotacao = templateManager.container.getWidget('widgetOldCot');
            
            setCotacao(response);
            widgetCotacao.activate();
            if (widgetCotacao.dataSource.data.VRPRECUNIAP == 0) {
                MessageService.showMessageByCode("PRECO_UNIT_INFORMADO");
            }
        });
    };
    
    this.setCotacaoWithParamsWithoutActivateWidgetCotacao = function() {
        getCotacoes(setCotacao);
    };

    this.backFilter = function(widget) {
        ScreenService.confirmMessage("Deseja realizar uma nova apuração?", 'question', function(success) {
            widget.container.getWidget('PRODUTOS').dataSource.setData([]);
            widget.container.getWidget('widgetOldCot').dataSource.setData([]);
            widget.container.getWidget('STRIPE').currentRow = [];
            this.openFilter();
        }.bind(this), function(error) {});
    }; 

    this.updateApuracao = function() {
        let widgetCotacao = templateManager.container.getWidget('widgetOldCot');
        let row = this.getRow(widgetCotacao);

        if (row.IDITMSELAPUR == 'S') {
            row.CDCENTITMAP = getLocalVar('CDFILCENTCMP');

            General.callRestEngine("cmp_fornecimento_apuracaoEfetivacao#/updateMelhorCotacao", "Row", row).then(function(response) {
                this.setCotacaoWithParams();
            }.bind(this));
        } else {
            let data = widgetCotacao.dataSource.data;
            for (let i = 0; i < data.length; i++) {
                if (row.CDNMREGIAO == data[i].CDNMREGIAO) {
                    if (data[i].IDITMSELAPUR == 'S') {
                        return false;
                    }
                }
            }
            
            if (this.idjustificativa == 'S') {
                let widgetJustificativa = templateManager.container.getWidget('JUSTIFICATIVA');
                
                ScreenService.openPopup(widgetJustificativa);
            } else {
                row.CDCENTITMAP = getLocalVar('CDFILCENTCMP');

                General.callRestEngine("cmp_fornecimento_apuracaoEfetivacao#/updateMelhorCotacao", "Row", row).then(function(response) {
                    this.setCotacaoWithParams();
                }.bind(this));
            }
        }

        this.removeClass('.lineImportant', 'currentRow');
    };

    this.confirmarJustificativa = function(widget) {
        if (widget.isValid()) {
            let row = angular.copy(templateManager.container.getWidget('widgetOldCot').currentRow);
            row.OLD_CDCENTITMAP = getLocalVar('CDFILCENTCMP');

            General.callRestEngine("cmp_fornecimento_apuracaoEfetivacao#/updateMelhorCotacao", "Row", row).then(function(response) {
                ScreenService.closePopup();
                this.setCotacaoWithParams();
            }.bind(this));
        }
    };
    
    this.updateVrUnitario = function() {
        let widgetCotacao = templateManager.container.getWidget('widgetOldCot');
        ScreenService.confirmMessage("Deseja atualizar a cotação conforme novo preço informado?", 'question', function(success) {
            let row = {
                'ITEM': widgetCotacao.dataSource.data.filter(item => item.changed),
                'PARAMS': {
                    'CDOPERADOR': getLocalVar('CDOPERADOR'),
                    'NMCONTAINER': templateManager.container.name,
                    'NMOPERADOR': getLocalVar('NMOPERADOR'),
                    'NRORG': getLocalVar('NRORG')
                }
            };
            
            setVisibleActions(widgetCotacao, false, ['BACK_APURACAO']);

            General.callRestEngine('cmp_fornecimento_apuracaoEfetivacao#/updateVrUnitario', "Row", row).then(function(success) {
    			widgetCotacao.lastInlineEditingElement.find('input').blur();
                widgetCotacao.activate();
                
    			removeChangedPropertiesFromRow(widgetCotacao);
                actionsAfterInlineEditOperation(widgetCotacao);
                
                MessageService.successNotificationByCode("REG_SUCCESS_SAVE");
            }).catch(function(error) {
                widgetCotacao.rollbackInlineEdit();
                
                actionsAfterInlineEditOperation(widgetCotacao);
                
                ScreenService.showMessage(error.data ? error.data.error : messages.timeout, 'error', 'error');
            });
        }.bind(this));
    };
    
    this.realizaEscolhaManualDoPreco = function() {
        this.changeEventCodeWidgetAfterMoveRow();
    };

    this.apurar = function(widget) {
        ScreenService.confirmMessage("Todas as cotações selecionadas serão desmarcadas. Confirma operação?", 'question', function(success) {

            General.callRestEngine('cmp_fornecimento_apuracaoEfetivacao#/melhorCotacao', 'Row', {
                'P_CDCENTCMP': getLocalVar('CDFILCENTCMP'),
                'P_NRSEQAPUR': param
            }).then(function(response) {
                this.setCotacaoWithParams().then(function() {
                    General.callRestEngine('cmp_fornecimento_apuracaoEfetivacao#/verificaPrecoEconomicoZerado', 'Row', {
                        'ITENS': widget.dataSource.data
                    }).then(function(response) {
                        if (!Util.isEmptyOrBlank(response.messages)) {
                            ScreenService.showMessage(response.messages[0].message);
                        }
                    });
                });
            }.bind(this));
        }.bind(this));
    };

    this.efetivar = function(widget) {
        const filterWidget = templateManager.container.getWidget('FILTER');
        
        General.callRestEngine('cmp_fornecimento_apuracaoEfetivacao#/efetivar', 'Row', {
            'P_CDCENTCMP': getLocalVar('CDFILCENTCMP'),
            'P_NRSEQAPUR': param,
            'P_CDFILIOPELOG': getLocalVar('CDFILIAL'),
            'P_CDCOMPRADOR': templateManager.container.getWidget('FILTER').currentRow.CDOPERADOR,
            'P_CDOPERALANC': getLocalVar('CDOPERADOR'),
            'P_DTAUTOFORN': getLocalVar('DTPROCESSA')
        }).then(function(response) {
            var data = response.dataset.data[0];
            const mensagens = data.MENSAGENS;
            // const inconsistencias = data.INCONSISTENCIAS;
            
            if (!Util.isEmptyOrBlank(mensagens)) {
                ScreenService.showMessage(mensagens[0]);
            }

            MessageService.successNotificationByCode("EFETIVACAO_SUCCESS");
            
            ScreenService.openWindow('cmp_fornecimento_autorizacaoFornecimento#sup05100_analise_AF').then(function() {
                
                templateManager.container.getWidget('widgetPrincipal').NRSEQAPUR = data.NRSEQAPUR[0];
                templateManager.container.getWidget('widgetPrincipal').CDFILLANC = getLocalVar('CDFILIAL');
                templateManager.container.getWidget('widgetPrincipal').CDFILCENTCMP = getLocalVar('CDFILCENTCMP');
                templateManager.container.getWidget('widgetPrincipal').CODCOMPRADOR = filterWidget.currentRow.CDCOMPRADOR;
                templateManager.container.getWidget('widgetPrincipal').CDOPERADOR = getLocalVar('CDOPERADOR');
                templateManager.container.getWidget('widgetPrincipal').DTAPURPREC = getLocalVar('DTPROCESSA');
                templateManager.container.getWidget('widgetPrincipal').DTINIAPUPREC = filterWidget.currentRow.DTENTREGA.formattedStart;
                templateManager.container.getWidget('widgetPrincipal').DTFINAPUPREC = filterWidget.currentRow.DTENTREGA.formattedEnd;
                templateManager.container.getWidget('widgetPrincipal').dataSource.data = [];
		    }.bind(this), function(error) {
			    throw error;
		    });
            
        }).catch(function(error) {
            if(error.data.error) {
                ScreenService.showMessage(error.data.error, 'error', 'error');
            } else {
                ScreenService.showMessage(error.data, 'error', 'error');
            }
        });
    };

    /*************************************************************************************/
    /*                                 FUNÇÕES POR ENTREGA                               */
    /*************************************************************************************/
    
    function executeDataSourcePorEntrega(row) {
        const filter = [
            {
                "name": "P_CDCENTITMAP",
                "operator": "=",
                "value": row.CDCENTITMAP
			},
            {
                "name": "P_NRSEQITMAP",
                "operator": "=",
                "value": row.NRSEQITMAP
			},
            {
                "name": "P_CDPRODITMAP",
                "operator": "=",
                "value": row.CDPRODITMAP
			},
            {
                "name": "P_CDREGITMAP",
                "operator": "=",
                "value": row.CDREGITMAP
			}
		];
		
        return General.callRestEngine('cmp_fornecimento_apuracaoEfetivacao#/crud/porEntrega', 'FilterData', filter);
    }
    
    function executeDataSourceAllEntregas(row) {
        const filter = [
            {
                "name": "P_CDCENTITMAP",
                "operator": "=",
                "value": row.CDCENTITMAP
			},
            {
                "name": "P_NRSEQITMAP",
                "operator": "=",
                "value": row.NRSEQITMAP
			},
            {
                "name": "P_CDPRODITMAP",
                "operator": "=",
                "value": row.CDARVPROD
			},
            {
                "name": "P_CDREGITMAP",
                "operator": "=",
                "value": row.CDREGITMAP
			}
		];
		
        return General.callRestEngine('cmp_fornecimento_apuracaoEfetivacao#/crud/allEntregas', 'FilterData', filter);
    }
    
    this.openPorEntrega = function() {
        let widgetCotacao = templateManager.container.getWidget('widgetOldCot');
        let row = this.getRow(widgetCotacao);

        if (row.IDITMSELAPUR == 'S' && widgetCotacao.getMode() !== 'inline_edit') {
            executeDataSourcePorEntrega(row).then(function(response) {
			    let widgetPorEntrega = templateManager.container.getWidget('POR_ENTREGA');
			    
                widgetPorEntrega.dataSource.setData(response.dataset.porEntrega);
                ScreenService.openSwipe(widgetPorEntrega).then(function(){
    				$('.swipe-tip').before('<div style="position: absolute;left: 0;top: 0;bottom: 0;right: 0;opacity: 0.6;transform: translate(-100%, 0);-webkit-transform: translate(-100%, 0);"></div>');
                    this.removeClass('.lineImportant', 'currentRow');
    			}.bind(this));
            }.bind(this));
        }
        else {
            ScreenService.closeSwipe();
        }
    };
    
    this.openPopupIncluir = function() {
        executeDataSourceAllEntregas(templateManager.container.getWidget('widgetOldCot').currentRow).then(function(response) {
            const widgetIncluir = templateManager.container.getWidget('INCLUIR');
            
            allEntregas = response.dataset.allEntregas;
            
            resetAllFieldsWidgetIncluir(widgetIncluir);
            setDataSourceFieldDataEntrega(widgetIncluir);
            ScreenService.openPopup(widgetIncluir);
            widgetIncluir.newRow();
        });
    };
    
    /**
     * Decodifica informação recebida como código na aba 'POR ENTREGA' e 
     * exibe em popup.
     */
    this.openInformacao = function() {
        let widgetPorEntrega = templateManager.container.getWidget('POR_ENTREGA');
        
        if (podeAbrirInformacao) {
            let widgetInformacao = templateManager.container.getWidget('INFORMACAO');
            let currentRowProduto = this.getRow(widgetPorEntrega);
            let idmotprescaf = currentRowProduto.IDMOTIVPRESC;
            let indexMensagem = '';
            let mensagem = '';
        
            for (var i = 0; i < idmotprescaf.length; i++) {
                indexMensagem = i + idmotprescaf[i];
                mensagem = mensagem + mensagensInformacao(indexMensagem);
            }
        
            if (!mensagem) {
                mensagem = mensagem + mensagensInformacao('M0');
            }
        
            widgetInformacao.currentRow.mensagem = mensagem;
        
            ScreenService.openPopup(widgetInformacao);
            podeAbrirInformacao = false;
        } else {
            let widgetItemPorEntrega = templateManager.container.getWidget('ITEM_POR_ENTREGA');
            
            transferRowData(widgetPorEntrega.currentRow, widgetItemPorEntrega.currentRow);
            
            ScreenService.openSwipe(widgetItemPorEntrega).then(function() {
                widgetItemPorEntrega.view();
            });
        }
    };
    
    function mensagensInformacao(indexMensagem, param) {
        var mensagens = {
            '01': 'Escolhido por Produto Preferencial.\n',
            '17': 'Escolhido por Fornecedor/Marca Obrigatórios.\n',
            '18': 'Escolhido por Fornecedor/Marca Preferenciais.\n',
            '19': 'Escolhido por Fornecedor/Marca Restritivos.\n',
            '24': 'Escolhido por Fornecedor Obrigatório.\n',
            '25': 'Escolhido por Fornecedor Preferencial.\n',
            '26': 'Escolhido por Fornecedor Restritivo.\n',
            '2A': 'Escolhido por Acordo.\n',
            '31': 'Escolhido por Marca Obrigatória.\n',
            '32': 'Escolhido por Marca Preferencial.\n',
            '33': 'Escolhido por Marca Restritiva.\n',
            '41': 'Escolhido da Lista de Preços.\n',
            '42': 'Escolhido da Cotação Fora da Lista.\n',
            '51': 'Apuração para solicitações extras - Fornecedor escolhido não é extra.\n',
            '52': 'Apuração para solicitações extras - Fornecedor escolhido é extra.\n',
            '53': 'Apuração para solicitações não extras.\n',
            'M0': 'Escolhida manualmente.\n',
            'M1': 'Cotação: ' + param + '\n',
            'M2': 'Lista de preços: ' + param + '\n',
            'M3': 'Obs.: Item alterado manualmente.\n'
        };
        return mensagens[indexMensagem] ? mensagens[indexMensagem] : '';
    }
    
    function resetAllFieldsWidgetIncluir(widgetIncluir) {
        widgetIncluir.currentRow = {};
        
        lockField(widgetIncluir.getField('NMFILIAL'));
        lockField(widgetIncluir.getField('NMFANTFORN'));
    }
    
    function getCotacoesLoaded() {
        let widgetCotacao = templateManager.container.getWidget('widgetOldCot');
        let data = widgetCotacao.dataSource.data;
        let values = [];
        let i = 0;
        
        data.forEach(function(value, index) {
            if (data[index].CDNMREGIAO == widgetCotacao.currentRow.CDNMREGIAO && data[index].CDNMPRODUTO == widgetCotacao.currentRow.CDNMPRODUTO) {
                values[i] = angular.copy(data[index]);
                i++;
            }
        }.bind(this));
        
        return values;
    }
    
    function getDate(date) {
        return date.split(' ')[0];
    }
    
    function dateEquals(dateFromItem, dateToBeEqual) {
        return getDate(dateFromItem) === dateToBeEqual;
    }
    
    function unidadeEquals(codigoUnidadeItem, codigoUnidadeToBeEqual) {
        return codigoUnidadeItem === codigoUnidadeToBeEqual;
    }
    
    function mountObjectWithDate(item) {
        return { DTENTRAPUT: getDate(item.DTENTRAPUT) };
    }
    
    function mountObjectWithFilial(item) {
        return { CDFILIAL: item.CDFILIAP, NMFILIAL: item.NMFILIAL };
    }
    
    function getArrayWithUniqueElements(dataToFilter, filterFunction, mountFunction, primitivePropertyToSearch) {
        let dataPrepared;
        
        if (typeof(filterFunction) === 'function') {
            dataPrepared = dataToFilter.filter(filterFunction);
        } else {
            dataPrepared = dataToFilter;
        }
        
        const result = [];
        const map = new Map();
        for (const item of dataPrepared) {
            if(!map.has(item[primitivePropertyToSearch])){
                map.set(item[primitivePropertyToSearch], true); // set any value to Map
                result.push(mountFunction(item));
            }
        }
        
        return result;
    }
    
    function setDataSourceFieldDataEntrega(widgetIncluir) {
        widgetIncluir.getField('DTENTRAPUT').dataSource.setData(getArrayWithUniqueElements(allEntregas, null, mountObjectWithDate, 'DTENTRAPUT'));
    }
    
    function setDataSourceFieldUnidade(fieldUnidade) {
        const filterFunction = function(item) {
            return dateEquals(item.DTENTRAPUT, fieldUnidade.getParent().currentRow.DTENTRAPUT);
        };
        
        fieldUnidade.dataSource.setData(getArrayWithUniqueElements(allEntregas, filterFunction, mountObjectWithFilial, 'CDFILIAP'));
    }
    
    function getQtdeComprarPorEntrega(date, codigoUnidade) {
        const mountFunction = function(item) { return item.QTSOLACOMP; };
        const dateAndUnidadeEqualsTo = function(item) {
            return dateEquals(item.DTENTRAPUT, date) && unidadeEquals(item.CDFILIAP, codigoUnidade);
        };
        
        return getArrayWithUniqueElements(allEntregas, dateAndUnidadeEqualsTo, mountFunction)[0];
    }
    
    function deepClearField(field) {
        field.clearValue();
    }
    
    function clearFieldWithDataSource(field) {
        deepClearField(field);
        field.dataSource.clearCheckedRows();
		field.dataSource.setData([]);
    }
    
    function lockField(field) {
        field.readOnly = true;
        field.validations = {};
    }
    
    function unlockField(field) {
        field.readOnly = false;
        field.validations = { required: true };
    }
    
    this.fieldOnChangeDataEntrega = function() {
        const widgetIncluir = templateManager.container.getWidget('INCLUIR');
        const fieldQtdeComprar = widgetIncluir.getField('QTSOLACOMP');
        const fieldUnidade = widgetIncluir.getField('NMFILIAL');
        const fieldFornecedor = widgetIncluir.getField('NMFANTFORN');
        
        fieldQtdeComprar.clearValue();
        
        clearFieldWithDataSource(fieldUnidade);
        unlockField(fieldUnidade);
        
        clearFieldWithDataSource(fieldFornecedor);
        lockField(fieldFornecedor);
        
        setDataSourceFieldUnidade(fieldUnidade);
    };
    
    function fornecedorEquals(fornecedor, fornecedorToBeFound) {
        return fornecedor.NRINSJURFORN === fornecedorToBeFound.NRINSJURFORN &&
               fornecedor.NRCOLEITMAP === fornecedorToBeFound.NRCOLEITMAP &&
               fornecedor.CDMARCITMAP === fornecedorToBeFound.CDMARCITMAP;
    }
    
    function filtraDataSourceFornecedor() {
        const widgetIncluir = templateManager.container.getWidget('INCLUIR');
        const widgetPorEntrega = templateManager.container.getWidget('POR_ENTREGA');
        const fieldFornecedor = widgetIncluir.getField('NMFANTFORN');
        const dataFilterByDateAndUnidade = widgetPorEntrega.dataSource.data.filter(item => {
            return dateEquals(item.DTENTRAPUT, widgetIncluir.currentRow.DTENTRAPUT) && unidadeEquals(item.CDFILIAP, widgetIncluir.currentRow.CDFILIAP);
        });
        const dataFilterByFornecedorNaoIncluido = fieldFornecedor.dataSource.data.filter(item => {
            return !dataFilterByDateAndUnidade.find(element => fornecedorEquals(element, item));
        });
        
        fieldFornecedor.dataSource.setData(dataFilterByFornecedorNaoIncluido);
    }
    
    this.fieldOnChangeUnidade = function() {
        const widgetIncluir = templateManager.container.getWidget('INCLUIR');
        const fieldQtdeComprar = widgetIncluir.getField('QTSOLACOMP');
        const fieldUnidade = widgetIncluir.getField('NMFILIAL');
        const fieldFornecedor = widgetIncluir.getField('NMFANTFORN');
        
        clearFieldWithDataSource(fieldFornecedor);
        
        if (Util.isEmptyOrBlank(fieldUnidade.value())) {
            fieldQtdeComprar.clearValue();
            lockField(fieldFornecedor);
        } else {
            unlockField(fieldFornecedor);
            
            fieldFornecedor.dataSource.setData(getCotacoesLoaded());
            filtraDataSourceFornecedor();
            
            if (Util.isEmptyOrBlank(fieldFornecedor.dataSource.data)) {
                fieldQtdeComprar.clearValue();
                MessageService.showMessageByCode("SEM_COTACAO_PROD");
            } else {
                fieldQtdeComprar.setValue(getQtdeComprarPorEntrega(widgetIncluir.currentRow.DTENTRAPUT, widgetIncluir.currentRow.CDFILIAP));
            }
        }
    };

    this.incluiFornecedor = function() {
        const widgetIncluir = templateManager.container.getWidget('INCLUIR');
        
        if (widgetIncluir.isValid()) {
            const widgetCotacao = templateManager.container.getWidget('widgetOldCot');
            const widgetPorEntrega = templateManager.container.getWidget('POR_ENTREGA');
            const currentRowIncluir = angular.copy(widgetIncluir.currentRow);
    
            currentRowIncluir.CDCENTITMAP = getLocalVar('CDFILCENTCMP');
            
            const row = {
                NEW_ITEM: currentRowIncluir,
                OLD_ITEM: widgetIncluir.backupRow || currentRowIncluir
            };
    
            General.callRestEngine("cmp_fornecimento_apuracaoEfetivacao#/incluirPorEntrega", "Row", row).then(function(response) {
                ScreenService.closePopup();
                MessageService.successNotificationByCode("REG_SUCCESS_SAVE");
                
                this.setCotacaoWithParamsWithoutActivateWidgetCotacao();
                
                executeDataSourcePorEntrega(widgetCotacao.currentRow).then(function(response) {
                    widgetPorEntrega.dataSource.setData(response.dataset.porEntrega);
                });
            }.bind(this))
            .catch(function(error) {
                ScreenService.showMessage(error.data ? error.data.error : messages.timeout, 'error', 'error');
            });
        }
    };
    
    this.saveEdicaoPorEntrega = function() {
        const widgetItemPorEntrega = templateManager.container.getWidget('ITEM_POR_ENTREGA');
        
        if (widgetItemPorEntrega.isValid()) {
            const widgetPorEntrega = templateManager.container.getWidget('POR_ENTREGA');
            const widgetCotacao = templateManager.container.getWidget('widgetOldCot');
            const row = {
                NEW_ITEM: widgetItemPorEntrega.currentRow,
                OLD_ITEM: widgetItemPorEntrega.backupRow
            };
            
            General.callRestEngine('cmp_fornecimento_apuracaoEfetivacao#/saveEdicaoPorEntrega', 'Row', row).then(function(response) {
    		    MessageService.successNotificationByCode("REG_SUCCESS_UPDATE");
    		    
    		    executeDataSourcePorEntrega(widgetCotacao.currentRow).then(function(response) {
                    widgetPorEntrega.dataSource.setData(response.dataset.porEntrega);
                });
    		    
    		    widgetItemPorEntrega.view();
            }).catch(function(error) {
                ScreenService.showMessage(error.data ? error.data.error : messages.timeout, 'error', 'error');
            });
        }
    };
    
    this.cancelEdicaoPorEntrega = function() {
        let widgetItemPorEntrega = templateManager.container.getWidget('ITEM_POR_ENTREGA');
        
		widgetItemPorEntrega.view();
		
		transferRowData(widgetItemPorEntrega.backupRow, widgetItemPorEntrega.currentRow);
    };
    
    this.editarEntrega = function() {
        let widgetItemPorEntrega = templateManager.container.getWidget('ITEM_POR_ENTREGA');
        
        General.edit(widgetItemPorEntrega);
    };

    /*************************************************************************************/
    /*                            FUNÇÕES PARA TRATATIVAS DE ERROS                       */
    /*************************************************************************************/
    
    this.confirmMessageModified = function(message, callbackForError) {
        let actions = [
            {
                label: ScreenService.i18n("Ok"),
                "default": true,
                code: function(e, deferred) {
                    deferred.resolve(e);
                }
            }
        ];
        
        if (Array.isArray(callbackForError.data.error) && !Util.isEmptyOrBlank(callbackForError.data.error)) {
            actions.push({
                label: ScreenService.i18n("Detalhes"),
                code: function(e, deferred) {
                    deferred.reject(callbackForError);
                }
            });
        }
        
        return ScreenService.showCustomDialog(message, 'error', 'error', actions);
    };
    
    /**
     * Altera a action 'Exportar' da widget de log de erros, passando uma
     * mensagem modificada, contendo:
     * - cabeçalho contendo o filtro realiazdo;
     * - os erros encontrados;
     */
    function atualizaExportar() {
        let filterWidget = templateManager.container.getWidget('FILTER');
        let messagesWidget = templateManager.container.getActiveWidget();
        let exportarAction = messagesWidget.getAction('exportar');
    
        messagesWidget.headerDataset = [];
    
        messagesWidget.headerDataset.push('-----------------------------------------');
        filterWidget.fields.forEach((field) => {
            if (field.isVisible === true) {
                let fieldValue;
                
                switch(field.name) {
                    case 'DTENTREGA':
                        let tempFieldValue = field.value();
                    
                        fieldValue =  tempFieldValue.start + ' a ' + tempFieldValue.end;
                        break;
                        
                    case 'EXTRA':
                        let tempField = field.dataSource.data.filter(dataItem => { return dataItem.value == field.value(); })[0];
                        
                        fieldValue = tempField.name;
                        break;
                        
                    default:
                        fieldValue = field.value();
                        break;
                }
                
                messagesWidget.headerDataset.push(field.label + ': ' + fieldValue);
            }
        });
        messagesWidget.headerDataset.push('-----------------------------------------');
    
        messagesWidget.bodyMessageDataset = messagesWidget.headerDataset.concat(messagesWidget.messagesDataset);
    
        exportarAction.events = [
            {
                "name": "ActionEvent",
                "code": "General.generateFileByDataSet(args.owner.widget.bodyMessageDataset);"
            }
        ];
    }
    
    this.handleInsertErrors = function (callback) {
        var filterWidget = templateManager.container.getWidget("FILTER");
        var message = messages.timeout;
        var errorMessages = Util.getNestedProperty(callback, ["data", "error"]);
    
        if (Array.isArray(errorMessages)) {
            filterWidget.reload().then(function() {
                let title = messages.errosApuracao;
                if (callback.data && callback.data.error) {
                    title = messages[callback.data.error] || title;
                }
                General.showErrorMessages(filterWidget, errorMessages, ScreenService.i18n(title));
                atualizaExportar();
            });
        } else {
            if (callback.data && callback.data.error) {
                message = messages[callback.data.error] || callback.data.error;
            }
            ScreenService.showMessage(message, "error");
        }
    };
    
    this.setFieldsVisibility = function(widget) {
    	const isReadOnly = getLocalVar('CDFILIAL') !== getLocalVar('CDFILCENTCMP');
    	const fieldFilial = widget.getField('LISTAFILIAL');
    	
    	widget.getField('CDREGIAO').readOnly = isReadOnly;
    	widget.getField('CDGRUPFILI').readOnly = isReadOnly;
    	fieldFilial.readOnly = isReadOnly;
    	
    	if(isReadOnly) {
    	    setUnidadeDestinoDefault(fieldFilial);
    	}
    };
    
    function setUnidadeDestinoDefault(fieldFilial) {
        const codigoUnidade = getLocalVar('CDFILIAL');
        const inFilterExists = function (fieldFilial) {
            return fieldFilial.dataSourceFilter.find(filter => filter.operator === 'IN');
        };
        
        if (!inFilterExists(fieldFilial)) {
			fieldFilial.dataSourceFilter.push({
			    name: 'CDFILIAL',
			    operator: 'IN',
			    value: [ codigoUnidade ]
			});
        }
		
		fieldFilial.dataSource
		    .load(fieldFilial.dataSourceFilter)
		    .then(function() {
			    fieldFilial.setValue([codigoUnidade]);
				const row = fieldFilial.dataSource.getFirstRow();
				
				row.__isSelected = true;
				
				fieldFilial.dataSource.updateCheckedRows();
				
				if (inFilterExists(fieldFilial)) {
				    fieldFilial.dataSourceFilter = fieldFilial.dataSourceFilter.filter(filter => filter.operator !== 'IN');
				}
				
				fieldFilial.dataSource.load(fieldFilial.dataSourceFilter);
			});
		
    }
}

Configuration(function(ContextRegister) {
    ContextRegister.register('ApuracaoEfetivacaoController', ApuracaoEfetivacaoController);
});
